@@ -15,6 +15,7 @@ from molotov.worker import Worker
 class Runner(object):
     """Manages processes & workers and grabs results.
     """
+
     def __init__(self, args, loop=None):
         self.args = args
         self.console = self.args.shared_console
@@ -26,17 +27,32 @@ class Runner(object):
         self.statsd = None
         self._tasks = []
         self._procs = []
-        self._results = SharedCounters('WORKER', 'REACHED', 'RATIO', 'OK',
-                                       'FAILED', 'MINUTE_OK', 'MINUTE_FAILED')
+        self._results = SharedCounters(
+            "WORKER",
+            "REACHED",
+            "RATIO",
+            "OK",
+            "FAILED",
+            "MINUTE_OK",
+            "MINUTE_FAILED",
+            "MAX_WORKERS",
+            "SETUP_FAILED",
+            "SESSION_SETUP_FAILED",
+        )
         self.eventer = EventSender(self.console)
 
     def _set_statsd(self):
         if self.args.statsd:
-            self.statsd = get_statsd_client(self.args.statsd_address,
-                                            loop=self.loop)
+            self.statsd = get_statsd_client(self.args.statsd_address)
         else:
             self.statsd = None
 
+    def run_coro(self, coro):
+        if not self.loop.is_running():
+            raise Exception("Loop is not running")
+        future = asyncio.run_coroutine_threadsafe(coro, self.loop)
+        return future.result()
+
     def gather(self, *futures):
         return asyncio.gather(*futures, loop=self.loop, return_exceptions=True)
 
@@ -44,7 +60,7 @@ class Runner(object):
         return asyncio.ensure_future(coro, loop=self.loop)
 
     def __call__(self):
-        global_setup = get_fixture('global_setup')
+        global_setup = get_fixture("global_setup")
         if global_setup is not None:
             try:
                 global_setup(self.args)
@@ -56,7 +72,7 @@ class Runner(object):
         try:
             return self._launch_processes()
         finally:
-            global_teardown = get_fixture('global_teardown')
+            global_teardown = get_fixture("global_teardown")
             if global_teardown is not None:
                 try:
                     global_teardown()
@@ -72,7 +88,7 @@ class Runner(object):
 
         if args.processes > 1:
             if not args.quiet:
-                self.console.print('Forking %d processes' % args.processes)
+                self.console.print("Forking %d processes" % args.processes)
             jobs = []
             for i in range(args.processes):
                 p = multiprocessing.Process(target=self._process)
@@ -85,7 +101,7 @@ class Runner(object):
             async def run(quiet, console):
                 while len(self._procs) > 0:
                     if not quiet:
-                        console.print(self.display_results(), end='\r')
+                        console.print(self.display_results(), end="\r")
                     for job in jobs:
                         if job.exitcode is not None and job in self._procs:
                             self._procs.remove(job)
@@ -93,9 +109,11 @@ class Runner(object):
                 await self.console.stop()
                 await self.eventer.stop()
 
-            tasks = [self.ensure_future(self.console.display()),
-                     self.ensure_future(self._send_workers_event(1)),
-                     self.ensure_future(run(args.quiet, self.console))]
+            tasks = [
+                self.ensure_future(self.console.display()),
+                self.ensure_future(self._send_workers_event(1)),
+                self.ensure_future(run(args.quiet, self.console)),
+            ]
             self.loop.run_until_complete(self.gather(*tasks))
         else:
             self._process()
@@ -115,13 +133,20 @@ class Runner(object):
         def _prepare():
             tasks = []
             delay = 0
-            if args.ramp_up > 0.:
+            if args.ramp_up > 0.0:
                 step = args.ramp_up / args.workers
             else:
-                step = 0.
+                step = 0.0
             for i in range(self.args.workers):
-                worker = Worker(i, self._results, self.console, self.args,
-                                self.statsd, delay, self.loop)
+                worker = Worker(
+                    i,
+                    self._results,
+                    self.console,
+                    self.args,
+                    self.statsd,
+                    delay,
+                    self.loop,
+                )
                 f = self.ensure_future(worker.run())
                 tasks.append(f)
                 delay += step
@@ -130,8 +155,8 @@ class Runner(object):
         if self.args.quiet:
             return _prepare()
         else:
-            msg = 'Preparing {} worker{}'
-            msg = msg.format(args.workers, 's' if args.workers > 1 else '')
+            msg = "Preparing {} worker{}"
+            msg = msg.format(args.workers, "s" if args.workers > 1 else "")
             return self.console.print_block(msg, _prepare)
 
     def _process(self):
@@ -139,10 +164,10 @@ class Runner(object):
 
         # coroutine that will kill everything when duration is up
         if self.args.duration and self.args.force_shutdown:
+
             async def _duration_killer():
                 cancelled = object()
-                res = await cancellable_sleep(self.args.duration,
-                                              result=cancelled)
+                res = await cancellable_sleep(self.args.duration, result=cancelled)
                 if res is cancelled or (res and not res.canceled()):
                     self._shutdown(None, None)
                     await asyncio.sleep(0)
@@ -158,10 +183,12 @@ class Runner(object):
             asyncio.set_event_loop(self.loop)
 
         if self.args.debug:
-            self.console.print('**** RUNNING IN DEBUG MODE == SLOW ****')
+            self.console.print("**** RUNNING IN DEBUG MODE == SLOW ****")
             self.loop.set_debug(True)
 
         self._set_statsd()
+        if self.statsd is not None:
+            self._tasks.append(self.ensure_future(self.statsd.connect()))
 
         if self.args.original_pid == os.getpid():
             self._tasks.append(self.ensure_future(self._send_workers_event(1)))
@@ -186,10 +213,9 @@ class Runner(object):
         try:
             self.loop.run_until_complete(self.gather(*self._tasks))
         finally:
-            self._kill_tasks()
             if self.statsd is not None:
-                self.statsd.close()
-            self.loop.run_until_complete(self.ensure_future(asyncio.sleep(0)))
+                self.loop.run_until_complete(self.ensure_future(self.statsd.close()))
+            self._kill_tasks()
             self.loop.close()
 
     def _kill_tasks(self):
@@ -202,19 +228,19 @@ class Runner(object):
         self._tasks[:] = []
 
     def display_results(self):
-        ok, fail = self._results['OK'].value, self._results['FAILED'].value
-        workers = self._results['WORKER'].value
-        pat = 'SUCCESSES: %s | FAILURES: %s | WORKERS: %s'
+        ok, fail = self._results["OK"].value, self._results["FAILED"].value
+        workers = self._results["WORKER"].value
+        pat = "SUCCESSES: %s | FAILURES: %s | WORKERS: %s"
         return pat % (ok, fail, workers)
 
     async def _display_results(self, update_interval):
         while not is_stopped():
-            self.console.print(self.display_results(), end='\r')
+            self.console.print(self.display_results(), end="\r")
             await cancellable_sleep(update_interval)
         await self.console.stop()
 
     async def _send_workers_event(self, update_interval):
         while not self.eventer.stopped() and not is_stopped():
-            workers = self._results['WORKER'].value
-            await self.eventer.send_event('current_workers', workers=workers)
+            workers = self._results["WORKER"].value
+            await self.eventer.send_event("current_workers", workers=workers)
             await cancellable_sleep(update_interval)
