@@ -6,14 +6,14 @@ This Molotov script has 2 scenario
 from molotov import scenario
 
 
-_API = 'http://localhost:8080'
+_API = "http://localhost:8080"
 
 
 @scenario(weight=40)
 async def scenario_one(session):
     async with session.get(_API) as resp:
         res = await resp.json()
-        assert res['result'] == 'OK'
+        assert res["result"] == "OK"
         assert resp.status == 200
 
 
