@@ -28,14 +28,13 @@ from molotov.sharedcounter import SharedCounters
 
 HERE = os.path.dirname(__file__)
 
-skip_pypy = pytest.mark.skipif(PYPY,
-                               reason='could not make work on pypy')
-only_pypy = pytest.mark.skipif(not PYPY, reason='only pypy')
+skip_pypy = pytest.mark.skipif(PYPY, reason="could not make work on pypy")
+only_pypy = pytest.mark.skipif(not PYPY, reason="only pypy")
 
-if os.environ.get('HAS_JOSH_K_SEAL_OF_APPROVAL', False):
-    _TIMEOUT = 1.
+if os.environ.get("HAS_JOSH_K_SEAL_OF_APPROVAL", False):
+    _TIMEOUT = 1.0
 else:
-    _TIMEOUT = .2
+    _TIMEOUT = 0.2
 
 
 async def serialize(console):
@@ -45,14 +44,14 @@ async def serialize(console):
             res.append(console._stream.get(block=True, timeout=_TIMEOUT))
         except Empty:
             break
-    return ''.join(res)
+    return "".join(res)
 
 
-class HandlerRedirect(http.server.SimpleHTTPRequestHandler):
+class RequestHandler(http.server.SimpleHTTPRequestHandler):
     def do_GET(self):
         if self.path == "/redirect":
             self.send_response(302)
-            self.send_header('Location', '/')
+            self.send_header("Location", "/")
             self.end_headers()
             return
         if self.path == "/slow":
@@ -63,28 +62,39 @@ class HandlerRedirect(http.server.SimpleHTTPRequestHandler):
             except SystemExit:
                 pass
             return
-        return super(HandlerRedirect, self).do_GET()
+        return super(RequestHandler, self).do_GET()
+
+    def do_POST(self):
+        content_length = int(self.headers["Content-Length"])
+        body = self.rfile.read(content_length)
+        self.send_response(200)
+        self.send_header("Content-Type", "application/json")
+        self.end_headers()
+        self.wfile.write(body)
 
 
 def run_server(port=8888):
     """Running in a subprocess to avoid any interference
     """
+
     def _run():
         os.chdir(HERE)
         socketserver.TCPServer.allow_reuse_address = True
         attempts = 0
         httpd = None
+        error = None
 
         while attempts < 3:
             try:
-                httpd = socketserver.TCPServer(("", port), HandlerRedirect)
+                httpd = socketserver.TCPServer(("", port), RequestHandler)
                 break
-            except Exception:
+            except Exception as e:
+                error = e
                 attempts += 1
-                time.sleep(.1)
+                time.sleep(0.1)
 
         if httpd is None:
-            raise OSError("Could not start the coserver")
+            raise OSError("Could not start the coserver: %s" % str(error))
 
         def _shutdown(*args, **kw):
             httpd.server_close()
@@ -101,71 +111,71 @@ def run_server(port=8888):
 
     while time.time() - start < 5 and not connected:
         try:
-            conn = HTTPConnection('localhost', 8888)
+            conn = HTTPConnection("localhost", 8888)
             conn.request("GET", "/")
             conn.getresponse()
             connected = True
         except Exception:
-            time.sleep(.1)
+            time.sleep(0.1)
     if not connected:
         os.kill(p.pid, signal.SIGTERM)
-        p.join(timeout=1.)
-        raise OSError('Could not connect to coserver')
+        p.join(timeout=1.0)
+        raise OSError("Could not connect to coserver")
     return p
 
 
-_CO = {'clients': 0, 'server': None}
+_CO = {"clients": 0, "server": None}
 
 
 @contextmanager
 def coserver(port=8888):
-    if _CO['clients'] == 0:
-        _CO['server'] = run_server(port)
+    if _CO["clients"] == 0:
+        _CO["server"] = run_server(port)
 
-    _CO['clients'] += 1
+    _CO["clients"] += 1
     try:
         yield
     finally:
-        _CO['clients'] -= 1
-        if _CO['clients'] == 0:
-            os.kill(_CO['server'].pid, signal.SIGTERM)
-            _CO['server'].join(timeout=1.)
-            _CO['server'].terminate()
-            _CO['server'] = None
+        _CO["clients"] -= 1
+        if _CO["clients"] == 0:
+            os.kill(_CO["server"].pid, signal.SIGTERM)
+            _CO["server"].join(timeout=1.0)
+            _CO["server"].terminate()
+            _CO["server"] = None
 
 
 def _respkw():
     from aiohttp.helpers import TimerNoop
-    return {'request_info': None,
-            'writer': None,
-            'continue100': None,
-            'timer': TimerNoop(),
-            'auto_decompress': True,
-            'traces': [],
-            'loop': asyncio.get_event_loop(),
-            'session': None}
+
+    return {
+        "request_info": None,
+        "writer": None,
+        "continue100": None,
+        "timer": TimerNoop(),
+        "traces": [],
+        "loop": asyncio.get_event_loop(),
+        "session": None,
+    }
 
 
-def Response(method='GET', status=200, body=b'***'):
-    if util.IS_AIOHTTP2:
-        response = LoggedClientResponse(method, URL('/'))
-    else:
-        response = LoggedClientResponse(method, URL('/'), **_respkw())
+def Response(method="GET", status=200, body=b"***"):
+    response = LoggedClientResponse(method, URL("/"), **_respkw())
     response.status = status
-    response.reason = ''
+    response.reason = ""
     response.code = status
     response.should_close = False
-    response.headers = CIMultiDict({})
-    response.raw_headers = []
+    response._headers = CIMultiDict({})
+    response._raw_headers = []
 
     class Body:
         async def read(self):
             return body
 
         def unread_data(self, data):
-            if body == b'':
-                err = AttributeError("'EmptyStreamReader' object has no "
-                                     "attribute 'unread_data'")
+            if body == b"":
+                err = AttributeError(
+                    "'EmptyStreamReader' object has no " "attribute 'unread_data'"
+                )
                 raise err
             pass
 
@@ -175,7 +185,7 @@ def Response(method='GET', status=200, body=b'***'):
     return response
 
 
-def Request(url="http://127.0.0.1/", method='GET', body=b'***'):
+def Request(url="http://127.0.0.1/", method="GET", body=b"***"):
     request = LoggedClientRequest(method, URL(url))
     request.body = body
     return request
@@ -196,12 +206,12 @@ class TestLoop(unittest.TestCase):
         asyncio.set_event_loop_policy(self.policy)
 
     def get_args(self, console=None):
-        args = namedtuple('args', 'verbose quiet duration exception')
+        args = namedtuple("args", "verbose quiet duration exception")
         args.force_shutdown = False
-        args.ramp_up = .0
+        args.ramp_up = 0.0
         args.verbose = 1
         args.quiet = False
-        args.duration = 5
+        args.duration = 0.2
         args.exception = True
         args.processes = 1
         args.debug = True
@@ -209,14 +219,16 @@ class TestLoop(unittest.TestCase):
         args.console = True
         args.statsd = False
         args.single_mode = None
+        args.single_run = False
         args.max_runs = None
-        args.delay = .0
+        args.delay = 0.0
         args.sizing = False
-        args.sizing_tolerance = .0
+        args.sizing_tolerance = 0.0
         args.console_update = 0
         args.use_extension = []
         args.fail = None
         args.force_reconnection = False
+        args.disable_dns_resolve = False
 
         if console is None:
             console = SharedConsole(interval=0)
@@ -233,17 +245,28 @@ def async_test(func):
         asyncio.set_event_loop(loop)
         loop.set_debug(True)
         console = SharedConsole(interval=0)
-        results = SharedCounters('WORKER', 'REACHED', 'RATIO', 'OK',
-                                 'FAILED', 'MINUTE_OK', 'MINUTE_FAILED')
-        kw['loop'] = loop
-        kw['console'] = console
-        kw['results'] = results
+        results = SharedCounters(
+            "WORKER",
+            "REACHED",
+            "RATIO",
+            "OK",
+            "FAILED",
+            "MINUTE_OK",
+            "MINUTE_FAILED",
+            "MAX_WORKERS",
+            "SETUP_FAILED",
+            "SESSION_SETUP_FAILED",
+        )
+        kw["loop"] = loop
+        kw["console"] = console
+        kw["results"] = results
         try:
             loop.run_until_complete(cofunc(*args, **kw))
         finally:
             loop.stop()
             loop.close()
             asyncio.set_event_loop(oldloop)
+
     return _async_test
 
 
@@ -260,6 +283,7 @@ def dedicatedloop(func):
                 loop.stop()
                 loop.close()
             asyncio.set_event_loop(old_loop)
+
     return _loop
 
 
@@ -277,6 +301,7 @@ def dedicatedloop_noclose(func):
         finally:
             loop._close()
             asyncio.set_event_loop(old_loop)
+
     return _loop
 
 
@@ -320,5 +345,5 @@ def catch_sleep(calls=None):
         # forces a context switch
         await original(0)
 
-    with patch('asyncio.sleep', _slept):
+    with patch("asyncio.sleep", _slept):
         yield calls
